
const player = require('node-wav-player')
const fs = require('fs')
const path = require('path')
const os = require('os')

module.exports.type = 'startup'
module.exports.meta = {
  name: 'voicevox_test',
  description: 'Voicevoxを利用してレスを読み上げます',
  version: '1.0.3',
  needVersion: '0.24.7'
}

// voicevox_engine API Document
// https://voicevox.github.io/voicevox_engine/api/

const voicevox_url = 'http://127.0.0.1:50021'
const wavInterval = 200
const defaultSpeaker = '0'

let played = false

module.exports.mainScript = async (settings, tools) => {
  // const { config, userSettings } = settings
  const { emitter, syncplay } = tools
  const tmp_dir = path.join(os.tmpdir(), 'siki-voicevox')
  await fs.promises.mkdir(tmp_dir, { recursive: true })
  if (syncplay) {
    const createWav = async (text, i, option) => {
      let rate = option.rate || 1
      let speaker = option.speaker || '0'
      let ret = await fetch(`${voicevox_url}/audio_query?text=${encodeURI(text)}&speaker=${speaker}`, {
        method: "POST",
        headers: {
          'Content-Type': 'application/json'
        }
      })
      const q = await ret.json()
      q.speedScale = rate
      const sound_row = await fetch(`${voicevox_url}/synthesis?speaker=${speaker}&enable_interrogative_upspeak=true`, {
        method: "POST",
        headers: {
          'Content-Type': 'application/json',
          'accept': 'audio/wav',
          'responseType': "stream"
        },
        body: JSON.stringify(q)
      })
      const buffer = Buffer.from(await sound_row.arrayBuffer())
      const wav_path = path.join(tmp_dir, `${i}.wav`)
      await fs.promises.writeFile(wav_path, buffer)
      // wavファイルの再生時間
      let duration = Math.floor(1000 * buffer.readUInt32LE(40, 44) / buffer.readUInt32LE(28, 32))
      return {
        duration: duration,
        path: wav_path
      }
    }
    syncplay.setPlay(async (id, arr) => {
      let playtime = 0
      let files = []
      let error
      for (let data of arr) {
        try {
          let { res, rate } = data
          rate = 0.3 + rate / 4
          let text = res.body
          // text = text.replace(/<anchor.*?>.*?<\/anchor>/g, '')
          text = text.replace(/\s*<br>\s*/g, "\n")
          text = text.replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '')
          text = text.replace(/https?:\/\/\S+/g, ' ')
          text = text.trim()
          // テキストがちょっと長くなるとメモリ消費が膨大になるため分割して連続再生
          let textBuf = text.split(/\n/g).filter(x => x)

          const createWavPromises = textBuf.map((text, i) => createWav(text, i, { speaker: defaultSpeaker, rate: rate }));
          const wavs = await Promise.all(createWavPromises)
          for (let wav of wavs) {
            if (wav?.path) {
              files.push(wav.path)
              playtime += wav.duration + wavInterval
            }
          }
        } catch (e) {
          error = e.stack
        }
        if (played) {
          player.stop()
        }
        (async () => {
          for (let i = 0; i < files.length; i++) {
            await player.play({ path: files[i], sync: true })
            played = true
          }
        })()
      }
      return {
        playtime,
        error
      }
    })
    syncplay.setStop(async (id) => {
      try {
        player.stop()
      } catch (e) {

      }
    })
  }
}
